#!/usr/bin/env python
#
# Copyright 2015 Falldog Hsieh <falldog7@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import base
import shutil
from os.path import join


class TestImportHook(base.TestPyConcreteBase):
    def test_py_import_hook(self):
        import os
        assert os
        import csv
        assert csv

    def test_py_relative_import(self):
        import os
        sys.path.insert(0, os.path.join(base.ROOT_DIR, 'test', 'data'))
        from test.data.relative_import import main
        self.assertEqual(main.data, 'main')
        self.assertEqual(main.util.data, 'util')
        sys.path.pop(0)

    def test_pye_relative_import(self):
        src_path = join(base.ROOT_DIR, 'test', 'data', 'relative_import')
        shutil.copytree(src_path, join(self.tmp_dir, 'relative_import'))
        base.touch(join(self.tmp_dir, '__init__.py'))

        self.lib_compile_pye(self.tmp_dir, remove_py=True, remove_pyc=True)

        from relative_import import main
        self.assertEqual(main.data, 'main')
        self.assertEqual(main.util.data, 'util')





#!/usr/bin/env python
#
# Copyright 2015 Falldog Hsieh <falldog7@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import fnmatch
import os
import sys
import unittest
import argparse
import py_compile
import subprocess
from os.path import abspath, dirname, join, exists, isdir, isfile


CUR_DIR = dirname(abspath(__file__))

IGNORE_FILES = ['.', '..', '.git', '.svn', 'pyconcrete']


class PyConcreteError(Exception):
    pass


class PyConcreteAdmin(object):
    def __init__(self):
        self.parser = None
        self.args = None
        self.parse_arg()

    def parse_arg(self):
        """
        Example:
            compile file/dir to .pye
            pyconcrete-admin.py compile --source={file} --pye

            compile file/dir to .pyc
            pyconcrete-admin.py compile --source={file} --pyc
        """
        parser = argparse.ArgumentParser(description='PyConcreteAdmin')
        subparsers = parser.add_subparsers()

        # === compile === #
        parser_compile = subparsers.add_parser('compile', help='compile .pye')
        parser_compile.add_argument('-s', '--source', dest='sources', default=None, nargs='+',
                                    help='specific the sources to process, source could be file/dir')
        parser_compile.add_argument('--pye', dest='pye', action='store_true', help='process on .pye')
        parser_compile.add_argument('--pyc', dest='pyc', action='store_true', help='process on .pyc')
        parser_compile.add_argument('--remove-py', dest='remove_py', action='store_true',
                                    help='remove .py after compile pye')
        parser_compile.add_argument('--remove-pyc', dest='remove_pyc', action='store_true',
                                    help='remove .pyc after compile pye')
        parser_compile.add_argument('-i', '--ignore-file-list', dest='ignore_file_list', metavar='filename', nargs='+',
                                    default=tuple(), help='ignore file name list')
        parser_compile.add_argument('-v', '--verbose', action='store_true', help='verbose mode')
        parser_compile.set_defaults(func=self.compile)

        # === test === #
        parser_test = subparsers.add_parser('test', help='test pycocnrete')
        parser_test.add_argument('-m', '--module', dest='test_module', default=None, help='test on single module')
        parser_test.add_argument('-v', '--verbose', action='store_true', help='verbose mode')
        parser_test.set_defaults(func=self.test)

        # === release === #
        parser_release = subparsers.add_parser('release', help='release pyconcrete for pypi')
        parser_release.set_defaults(func=self.release)

        if len(sys.argv) == 1:
            parser.print_help()

        else:
            try:
                args = parser.parse_args()
                args.func(args)
            except PyConcreteError as e:
                print('Error: ' + str(e))
                sys.exit(1)

    def compile(self, args):
        if args.sources is None:
            raise PyConcreteError("arg: compile, need assign --source={file/dir} to process")
        if not args.pye and not args.pyc:
            raise PyConcreteError("arg: compile, need assign the type for compile to `pye` or `pyc`")

        for source in args.sources:
            if isfile(source):
                if not source.endswith('.py'):
                    raise PyConcreteError("source file should end with .py")

                if args.pye:
                    self._compile_pye_file(args, source)
                elif args.pyc:
                    self._compile_pyc_file(args, source)

            elif isdir(source):
                self._compile_dir(args, source)

    def _get_ignore_patterns(self, args):
        patterns = []
        for pat in args.ignore_file_list:
            if not pat.startswith("*"):
                if not pat.startswith(os.sep):
                    pat = os.sep + pat
                pat = "*" + pat
            patterns.append(pat)
        return patterns

    def _compile_dir(self, args, folder):
        # ignore patterns
        patterns = self._get_ignore_patterns(args)
        for file in os.listdir(folder):
            fullpath = join(folder, file)
            if (file in IGNORE_FILES or self._fnmatch(fullpath, patterns)):
                continue

            if isdir(fullpath):
                self._compile_dir(args, fullpath)
            elif fullpath.endswith('.py'):
                if args.pye:
                    self._compile_pye_file(args, fullpath)
                elif args.pyc:
                    self._compile_pyc_file(args, fullpath)

    def _compile_pyc_file(self, args, py_file):
        pyc_file = py_file + 'c'
        pyc_exists = exists(pyc_file)
        if not pyc_exists or os.stat(py_file).st_mtime != os.stat(pyc_file).st_mtime:
            py_compile.compile(py_file, cfile=pyc_file)
            if args.verbose:
                print('* create %s' % pyc_file)
        else:
            if args.verbose:
                print('* skip %s' % pyc_file)

        if args.remove_py:
            os.remove(py_file)

    def _compile_pye_file(self, args, py_file):
        """
        if there is no .pyc file, compile .pyc first
        then compile .pye
        """
        import pyconcrete
        pyc_file = py_file + 'c'
        pye_file = py_file + 'e'
        pyc_exists = exists(pyc_file)
        if not pyc_exists or os.stat(py_file).st_mtime != os.stat(pyc_file).st_mtime:
            py_compile.compile(py_file, cfile=pyc_file)
        if not exists(pye_file) or os.stat(py_file).st_mtime != os.stat(pye_file).st_mtime:
            pyconcrete.encrypt_file(pyc_file, pye_file)
            if args.verbose:
                print('* create %s' % pye_file)
        else:
            if args.verbose:
                print('* skip %s' % pye_file)

        # .pyc doesn't exists at beginning, remove it after .pye created
        if not pyc_exists or args.remove_pyc:
            os.remove(pyc_file)
        if args.remove_py:
            os.remove(py_file)

    def test(self, args):
        test_dir = join(CUR_DIR, 'test')
        sys.path.insert(0, test_dir)  # for loadTestsFromName

        if args.test_module:
            suite = unittest.TestLoader().loadTestsFromName(args.test_module)
        else:
            suite = unittest.TestLoader().discover(test_dir)

        verbosity = 2 if args.verbose else 1
        result = unittest.TextTestRunner(verbosity=verbosity).run(suite)
        if not result.wasSuccessful():
            sys.exit(1)

    def release(self, args):
        try:
            import pypandoc
            readme = pypandoc.convert('README.md', 'rst')
            with open('README.rst', 'wb') as f:
                f.write(readme.encode('utf8'))
        except ImportError:
            print('you need to install `pypandoc` before release pyconcrete')
            raise

        subprocess.call('python setup.py sdist', shell=True)  # ignore can't found README error
        subprocess.check_output('twine upload dist/*', shell=True)
        subprocess.check_output('rm README.rst', shell=True)
        subprocess.check_output('rm -rf build', shell=True)
        subprocess.check_output('rm -rf dist', shell=True)

    @staticmethod
    def _fnmatch(name, patterns):
        """Test whether FILENAME matches PATTERN.

        Patterns are Unix shell style:
        *       matches everything
        ?       matches any single character
        [seq]   matches any character in seq
        [!seq]  matches any char not in seq
        """
        return any(fnmatch.fnmatch(name, pat) for pat in patterns)


if __name__ == '__main__':
    admin = PyConcreteAdmin()
    # admin.run()



#!/usr/bin/env python
#
# Copyright 2015 Falldog Hsieh <falldog7@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys
import time
import unittest
from zipfile import ZipFile
from multiprocessing import Process, Queue
from os.path import dirname, abspath, join

import base

CUR_DIR = abspath(dirname(__file__))
ROOT_DIR = abspath(join(CUR_DIR, '..'))
DATA_DIR = join(CUR_DIR, 'data')
REQUEST_ZIP = join(DATA_DIR, 'requests-2.12.4.zip')
REQUEST_MAIN = join(DATA_DIR, 'main_requests.py')
PYADMIN_PATH = join(ROOT_DIR, 'pyconcrete-admin.py')
RUN_COUNT = int(os.environ.get('TEST_PYE_PERFORMANCE_COUNT', '5'))


def main_requests(import_concrete, q):
    """
    testing main function for multiprocessing
    purpose: testing import without exception
    """
    if import_concrete:
        import pyconcrete

    t = time.time()
    import requests

    from requests.adapters import HTTPAdapter
    from requests.auth import HTTPDigestAuth, _basic_auth_str
    from requests.compat import (Morsel, cookielib, getproxies, str, urljoin, urlparse, is_py3, builtin_str)
    from requests.cookies import cookiejar_from_dict, morsel_to_cookie
    from requests.exceptions import (
        ConnectionError, ConnectTimeout, InvalidSchema, InvalidURL, MissingSchema, ReadTimeout, Timeout, RetryError
    )
    from requests.models import PreparedRequest
    from requests.structures import CaseInsensitiveDict
    from requests.sessions import SessionRedirectMixin
    from requests.models import urlencode
    from requests.hooks import default_hooks

    t = time.time() - t
    q.put(requests.__file__)
    q.put(t)
    

@unittest.skipIf(not os.path.exists(REQUEST_ZIP), "requests zip file doesn't exists")
class TestPerformance(base.TestPyConcreteBase):
    
    def setUp(self):
        super(TestPerformance, self).setUp()

        zip = ZipFile(REQUEST_ZIP)
        zip.extractall(self.tmp_dir)
        zip.close()
        
        self.req_dir = join(self.tmp_dir, 'requests')
        base.touch(join(self.req_dir, '__init__.py'))

    def _test_requests(self, import_concrete):
        sys.path.insert(0, self.req_dir)
        
        q = Queue()
        p = Process(target=main_requests, args=(import_concrete, q))
        
        p.start()
        path = q.get(timeout=5)
        t = q.get(timeout=2)
        p.join()

        self.assertTrue(path.startswith(self.req_dir), "wrong import path of requests = %s" % path)
        return t

    def test_requests_pye(self):
        self.lib_compile_pye(self.req_dir, remove_py=True, remove_pyc=True)
        t = 0.0
        for i in range(RUN_COUNT):
            t += self._test_requests(True)
        print('test import request (pye) [count=%d] total time = %.2f, avg time = %.2f' % (RUN_COUNT, t, t/RUN_COUNT))

    def test_requests_pyc(self):
        self.lib_compile_pyc(self.req_dir, remove_py=True)
        t = 0.0
        for i in range(RUN_COUNT):
            t += self._test_requests(False)
        print('test import request (pyc) [count=%d] total time = %.2f, avg time = %.2f' % (RUN_COUNT, t, t/RUN_COUNT))

    def test_requests_pyc_with_import_hooker(self):
        self.lib_compile_pyc(self.req_dir, remove_py=True)
        t = 0.0
        for i in range(RUN_COUNT):
            t += self._test_requests(True)
        print('test import request (pyc) (import hooker) [count=%d] total time = %.2f, avg time = %.2f' % (RUN_COUNT, t, t/RUN_COUNT))

    def test_requests_py(self):
        t = 0.0
        for i in range(RUN_COUNT):
            t += self._test_requests(False)
        print('test import request (py) [count=%d] total time = %.2f, avg time = %.2f' % (RUN_COUNT, t, t/RUN_COUNT))

    def test_requests_py_with_import_hooker(self):
        t = 0.0
        for i in range(RUN_COUNT):
            t += self._test_requests(True)
        print('test import request (py) (import hooker) [count=%d] total time = %.2f, avg time = %.2f' % (RUN_COUNT, t, t/RUN_COUNT))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Copyright 2015 Falldog Hsieh <falldog7@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

import base
import unittest


class TestBasic(base.TestPyConcreteBase):
    def test_import(self):
        import pyconcrete

        print('pyconcrete info="%s", path=%s' % (pyconcrete.info(), pyconcrete.__file__))

        # print "lib dir=[%s], pyconcrete __init__=[%s]" % (self.lib_dir, pyconcrete.__file__)
        self.assertTrue(pyconcrete.__file__.startswith(self.lib_dir))  # check import correct module

    def test_import_pye(self):
        py_code = b"v_int=1\r\n"
        py_code += b"v_string='abc'\r\n"
        pye_filepath = self.lib_gen_pye(py_code, 'test_module.pye')

        # import test.pye testing
        import test_module

        self.assertEqual(pye_filepath, test_module.__file__)  # check import correct module
        self.assertEqual(1, test_module.v_int)
        self.assertEqual('abc', test_module.v_string)


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
#
# Copyright 2015 Falldog Hsieh <falldog7@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys
import shutil
import unittest
import subprocess
from os.path import dirname, abspath, join, exists

import base

ROOT_DIR = abspath(join(dirname(__file__), '..'))
SAMPLE_PACKAGE_DIR = join(ROOT_DIR, 'test', 'data')


class TestAdminScript(base.TestPyConcreteBase):
    def setUp(self):
        super(TestAdminScript, self).setUp()
        self._ori_dir = os.getcwd()
        os.chdir(ROOT_DIR)

    def tearDown(self):
        super(TestAdminScript, self).tearDown()
        os.chdir(self._ori_dir)

    def test_parse_file(self):
        target_dir = join(self.tmp_dir, 'data')
        shutil.copytree(SAMPLE_PACKAGE_DIR, target_dir)
        target_file = join(target_dir, 'main.py')
        expect_file = join(target_dir, 'main.pye')

        subprocess.check_call(
            '%s pyconcrete-admin.py compile --source=%s --pye --verbose' % (sys.executable, target_file),
            env=base.get_pyconcrete_env_path(),
            shell=True,
        )

        self.assertTrue(exists(expect_file))

    def test_parse_folder(self):
        target_dir = join(self.tmp_dir, 'data')
        # print 'src=%s, target=%s' % (SAMPLE_PACKAGE_DIR, target_dir)
        shutil.copytree(SAMPLE_PACKAGE_DIR, target_dir)
        expect_file1 = join(target_dir, '__init__.pye')
        expect_file2 = join(target_dir, 'main.pye')

        subprocess.check_call(
            '%s pyconcrete-admin.py compile --source=%s --pye --verbose' % (sys.executable, target_dir),
            env=base.get_pyconcrete_env_path(),
            shell=True,
        )

        self.assertTrue(exists(expect_file1))
        self.assertTrue(exists(expect_file2))


class TestAdminIgnoreFilesScript(base.TestPyConcreteBase):
    def setUp(self):
        super(TestAdminIgnoreFilesScript, self).setUp()
        self.lib_gen_py("", "test1.py")
        self.lib_gen_py("", "test2.py")
        self.lib_gen_py("", "test.py")
        self.lib_gen_py("", "a_test.py")
        self.lib_gen_py("", "b_test.py")

    def test_ignore_file_list_match(self):
        target_dir = join(self.tmp_dir, 'data')
        relative_import_dir = join(self.tmp_dir, 'data', 'relative_import')
        shutil.copytree(SAMPLE_PACKAGE_DIR, target_dir)

        expect_file1 = join(target_dir, 'main.pye')
        # relative_import/util.pye
        expect_file2 = join(relative_import_dir, 'util.pye')
        # relative_import/main.pye
        expect_file3 = join(relative_import_dir, 'main.pye')

        subprocess.check_call(
            ('%s pyconcrete-admin.py compile --source=%s --pye --verbose -i relative_import/util.py'
             % (sys.executable, target_dir)),
            env=base.get_pyconcrete_env_path(),
            shell=True,
        )

        self.assertTrue(exists(expect_file1))
        self.assertFalse(exists(expect_file2))
        self.assertTrue(exists(expect_file3))

    def test_ignore_file_list_match_everything(self):
        target_dir = join(self.tmp_dir, 'data')
        relative_import_dir = join(self.tmp_dir, 'data', 'relative_import')
        shutil.copytree(SAMPLE_PACKAGE_DIR, target_dir)

        expect_file1 = join(target_dir, 'main.pye')
        # relative_import/util.pye
        expect_file2 = join(relative_import_dir, 'util.pye')
        # relative_import/main.pye
        expect_file3 = join(relative_import_dir, 'main.pye')

        subprocess.check_call(
            ('%s pyconcrete-admin.py compile --source=%s --pye --verbose -i relative_import/*'
             % (sys.executable, target_dir)),
            env=base.get_pyconcrete_env_path(),
            shell=True,
        )

        self.assertTrue(exists(expect_file1))
        self.assertFalse(exists(expect_file2))
        self.assertFalse(exists(expect_file3))

    def test_ignore_file_list_match_everything_patterns(self):
        patterns = [
            "*%s%s" % (os.sep, "test.py"),  # */test.py
            "%s%s" % (os.sep, "test.py"),   # /test.py
            "test.py",                      # test.py
        ]
        expect_file1 = join(self.tmp_dir, 'test1.pye')
        expect_file2 = join(self.tmp_dir, 'test2.pye')
        expect_file3 = join(self.tmp_dir, 'test.pye')
        expect_file4 = join(self.tmp_dir, 'a_test.pye')
        expect_file5 = join(self.tmp_dir, 'b_test.pye')

        def test_pattern(pat):
            subprocess.check_call(
                ('%s pyconcrete-admin.py compile --source=%s --pye --verbose -i "%s"'
                 % (sys.executable, self.tmp_dir, pat)),
                env=base.get_pyconcrete_env_path(),
                shell=True,
            )

            msg = "pattern(%s) fail" % pat
            self.assertTrue(exists(expect_file1), msg)
            self.assertTrue(exists(expect_file2), msg)
            self.assertFalse(exists(expect_file3), msg)  # test.py excluded
            self.assertTrue(exists(expect_file4), msg)
            self.assertTrue(exists(expect_file5), msg)

        for pat in patterns:
            test_pattern(pat)

    def test_ignore_file_list_match_single(self):
        target_dir = join(self.tmp_dir, 'data')
        shutil.copytree(SAMPLE_PACKAGE_DIR, target_dir)

        expect_file1 = join(target_dir, '__init__.pye')
        expect_file2 = join(target_dir, 'main.pye')
        expect_file3 = join(self.tmp_dir, 'test1.pye')
        expect_file4 = join(self.tmp_dir, 'test2.pye')

        subprocess.check_call(
            ('%s pyconcrete-admin.py compile --source=%s --pye --verbose -i main.py test?.py'
             % (sys.executable, self.tmp_dir)),
            env=base.get_pyconcrete_env_path(),
            shell=True,
        )

        self.assertTrue(exists(expect_file1))
        self.assertFalse(exists(expect_file2))
        self.assertFalse(exists(expect_file3))
        self.assertFalse(exists(expect_file4))

    def test_ignore_file_list_match_in_seq(self):
        expect_file1 = join(self.tmp_dir, 'test1.pye')
        expect_file2 = join(self.tmp_dir, 'test2.pye')

        subprocess.check_call(
            ('%s pyconcrete-admin.py compile --source=%s --pye --verbose -i main.py test[1,3].py'
             % (sys.executable, self.tmp_dir)),
            env=base.get_pyconcrete_env_path(),
            shell=True,
        )

        self.assertFalse(exists(expect_file1))
        self.assertTrue(exists(expect_file2))

    def test_ignore_file_list_match_not_in_seq(self):
        expect_file1 = join(self.tmp_dir, 'test1.pye')
        expect_file2 = join(self.tmp_dir, 'test2.pye')

        subprocess.check_call(
            ('%s pyconcrete-admin.py compile --source=%s --pye --verbose -i main.py test[!1,3].py'
             % (sys.executable, self.tmp_dir)),
            env=base.get_pyconcrete_env_path(),
            shell=True,
        )

        self.assertTrue(exists(expect_file1))
        self.assertFalse(exists(expect_file2))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Copyright 2015 Falldog Hsieh <falldog7@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

import os
import base
import subprocess


class TestPyconcreteExe(base.TestPyConcreteBase):
    def test_execute_pye(self):
        pye = self.lib_gen_pye('import os\n'
                               'print(os.getcwd())',
                               pye_filename='main.pye')

        output = subprocess.check_output([self._pyconcrete_exe, pye])
        output = output.decode('utf8')
        self.assertEqual(output.strip(), os.getcwd())

    def test_execute_not_exist_file(self):
        output = subprocess.check_call([self._pyconcrete_exe, 'TheFileIsNotExists'])
        self.assertEqual(output, 0)

    def test_sys_path(self):
        """ pye dir should be listed in sys.path """
        pye = self.lib_gen_pye('import sys\n'
                               'paths = "\\n".join(sys.path)\n'
                               'print(paths)',
                               pye_filename='main.pye')

        pye_dir = os.path.dirname(pye)
        pye_dir = os.path.realpath(pye_dir)  # tmpdir would be under symlink at MacOS

        output = subprocess.check_output([self._pyconcrete_exe, pye])
        output = output.decode('utf8')
        output = output.replace('\r\n', '\n')
        paths = output.split('\n')

        self.assertTrue(pye_dir in paths, "pye dir(%s) not in output paths %s" % (pye_dir, paths))

    def test_sys_argv(self):
        pye = self.lib_gen_pye('import sys\n'
                               'argv = " ".join(sys.argv)\n'
                               'print(argv)',
                               pye_filename='main.pye')

        output = subprocess.check_output([self._pyconcrete_exe, pye])
        output = output.decode('utf8')
        self.assertEqual(output.strip(), pye)

    def test_sys_argv_more_arguments(self):
        pye = self.lib_gen_pye('import sys\n'
                               'argv = " ".join(sys.argv)\n'
                               'print(argv)',
                               pye_filename='main.pye')

        output = subprocess.check_output([self._pyconcrete_exe, pye, '1', '2', '3'])
        output = output.decode('utf8')

        self.assertEqual(output.strip(), pye + ' 1 2 3')

    # def test_sys_exit(self):
    #     pye = self.lib_gen_pye('import sys\n'
    #                            'sys.exit(1)',
    #                            pye_filename='main.pye')
    #
    #     output = subprocess.call([self._pyconcrete_exe, pye])
    #     self.assertEqual(output, 1)

    def test__name__be__main__(self):
        pye = self.lib_gen_pye('print(__name__)',
                               pye_filename='main.pye')

        output = subprocess.check_output([self._pyconcrete_exe, pye])
        output = output.decode('utf8')
        self.assertEqual(output.strip(), '__main__')




#!/usr/bin/env python
#
# Copyright 2015 Falldog Hsieh <falldog7@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys
import imp
import hashlib
import sysconfig
from os.path import join
from distutils.core import setup, Extension, Command
from distutils.dist import Distribution
from distutils.command.build import build
from distutils.command.build_ext import build_ext
from distutils.command.install import install
from src.config import DEFAULT_KEY, TEST_DIR, SRC_DIR, PY_SRC_DIR, EXT_SRC_DIR, EXE_SRC_DIR, SECRET_HEADER_PATH

version_mod = imp.load_source('version', join(PY_SRC_DIR, 'version.py'))
version = version_mod.__version__

PY2 = sys.version_info[0] < 3


# .rst should created by pyconcrete-admin
if os.path.exists('README.rst'):
    readme_path = 'README.rst'
else:
    readme_path = 'README.md'
with open(readme_path, 'r') as f:
    readme = f.read()

try:
   input = raw_input
except NameError:
   pass


def is_mingw():
    """Compiler is mingw."""
    dst = Distribution()
    dst.parse_config_files()
    if 'build' not in dst.command_options:
        return False
    _build = dst.command_options['build']
    if 'compiler' not in _build:
        return False
    compiler = _build['compiler']
    if compiler[1].startswith('mingw'):
        return True
    return False


def is_msvc():
    return sys.platform == 'win32' and not is_mingw()


def hash_key(key):
    if PY2:
        factor = sum([ord(s) for s in key])
    else:
        factor = sum([s for s in key])
    factor %= 128
    if factor < 16:
        factor += 16

    m = hashlib.md5()
    m.update(key)
    k = m.digest()

    return k, factor


def create_secret_key_header(key, factor):
    # reference from - http://stackoverflow.com/questions/1356896/how-to-hide-a-string-in-binary-code
    # encrypt the secret key in binary code
    # avoid to easy read from HEX view

    key_val_lst = []
    for i, k in enumerate(key):
        n = ord(k) if PY2 else k
        key_val_lst.append("(0x%X ^ (0x%X - %d))" % (n, factor, i))
    key_val_code = ", ".join(key_val_lst)

    code = """
        #define SECRET_NUM 0x%X
        #define SECRET_KEY_LEN %d
        static const unsigned char* GetSecretKey()
        {
            unsigned int i = 0;
            static unsigned char key[] = {%s, 0/* terminal char */};
            static int is_encrypt = 1/*true*/;
            if( is_encrypt )
            {
                for(i = 0 ; i < SECRET_KEY_LEN ; ++i)
                {
                    key[i] = key[i] ^ (SECRET_NUM - i);
                }
                is_encrypt = 0/*false*/;
            }
            return key;
        }
    """ % (factor, len(key), key_val_code)

    with open(SECRET_HEADER_PATH, 'w') as f:
        f.write(code)


def remove_secret_key_header():
    if os.path.exists(SECRET_HEADER_PATH):
        os.remove(SECRET_HEADER_PATH)


class ExeDistribution(Distribution):
    exe_modules = ()

    def has_exe_modules(self):
        return self.exe_modules and len(self.exe_modules) > 0


# ================================================= command ================================================= #


class CmdBase:
    def pre_process(self):
        self.manual_create_secrete_key_file = not os.path.exists(SECRET_HEADER_PATH)
        if self.manual_create_secrete_key_file:
            if not self.passphrase:
                self.passphrase = input("please input the passphrase \nfor encrypt your python script (enter for default) : \n")
                if len(self.passphrase) == 0:
                    self.passphrase = DEFAULT_KEY
                else:
                    passphrase2 = input("please input again to confirm\n")
                    if self.passphrase != passphrase2:
                        raise Exception("Passphrase is different")

            k, f = hash_key(self.passphrase.encode('utf8'))
            create_secret_key_header(k, f)

    def post_process(self):
        if self.manual_create_secrete_key_file:
            remove_secret_key_header()


class BuildEx(CmdBase, build):
    """
    execute extra function before/after build.run()
    """
    user_options = build.user_options + [('passphrase=', None, 'specify passphrase')]

    def initialize_options(self):
        build.initialize_options(self)
        self.passphrase = None

    def run(self):
        self.pre_process()
        ret = build.run(self)
        self.post_process()
        return ret


class BuildExe(CmdBase, build_ext):
    """
    execute extra function before/after build.run()
    """
    user_options = build.user_options + [('passphrase=', None, 'specify passphrase')]

    def initialize_options(self):
        build_ext.initialize_options(self)
        self.passphrase = None
        self.build_scripts = None

    def finalize_options(self):
        build_ext.finalize_options(self)
        self.set_undefined_options('build',
                                   ('build_scripts', 'build_scripts'))
        self.extensions = self.distribution.exe_modules

    def run(self):
        self.pre_process()
        build_ext.run(self)
        self.post_process()

    def build_extensions(self):
        """
        copy from distutils/command/build_ext.py build_ext.build_extensions
        just override build_extensions for covert the compiler setup from build_ext.run()
        """
        ext = self.extensions[0]  # only one exe extension

        macros = ext.define_macros[:]
        for undef in ext.undef_macros:
            macros.append((undef,))

        objects = self.compiler.compile(
            list(ext.sources),
            output_dir=self.build_temp,
            include_dirs=ext.include_dirs,
            macros=macros,
            debug=self.debug,
            depends=ext.depends)

        self._built_objects = objects[:]

        # link for executable file
        self.compiler.link_executable(
            objects,
            ext.name,
            libraries=self.get_libraries(ext),
            library_dirs=ext.library_dirs,
            runtime_library_dirs=ext.runtime_library_dirs,
            extra_postargs=ext.extra_link_args,
            output_dir=self.build_scripts,  # should not use build_lib, it will be copied to python site-packages
            debug=self.debug)


class InstallEx(CmdBase, install):
    """
    execute extra function before/after install.run()
    """
    user_options = install.user_options + [('passphrase=', None, 'specify passphrase')]
    sub_commands = install.sub_commands + [('build_exe', lambda self: True)]

    def initialize_options(self):
        install.initialize_options(self)
        self.passphrase = None
        self.build_scripts = None

    def finalize_options(self):
        install.finalize_options(self)
        self.set_undefined_options('build',
                                   ('build_scripts', 'build_scripts'))

    def run(self):
        self.pre_process()
        ret = install.run(self)
        self.install_exe()
        self.post_process()
        self.create_pth()
        return ret

    def create_pth(self):
        """
        create pyconcrete.pth for automatic import pyconcrete after python launched
        """
        # setup(
        #     data_files=(sysconfig.get_python_lib(), 'pyconcrete.pth'),
        #     ...
        # )
        # it will got wrong result in ubuntu, get_python_lib got /usr/lib
        # but pyconcrete installed in /usr/local/lib
        #
        # get install_libbase to avoid this issue
        filename = join(self.install_libbase, 'pyconcrete.pth')
        with open(filename, 'w') as f:
            f.write('import pyconcrete')
        print('creating %s' % filename)

    def install_exe(self):
        if sys.platform == 'win32':
            # install `pyconcrete.exe` to %PYTHON%/Scripts
            exe_name = 'pyconcrete.exe'
        else:
            # install `pyconcrete` to /usr/local/bin
            exe_name = 'pyconcrete'
        self.copy_file(os.path.join(self.build_scripts, exe_name),
                       os.path.join(self.install_scripts, exe_name))

# ================================================= test command ================================================= #


class TestEx(Command):
    description = "Running all unit test for pyconcrete"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import unittest
        suite = unittest.TestLoader().discover(TEST_DIR)
        unittest.TextTestRunner(verbosity=2).run(suite)


# ================================================= extension ================================================= #

def get_include_dirs():
    openaes_include_dirs = [
        join(EXT_SRC_DIR),
        join(EXT_SRC_DIR, 'openaes', 'inc'),
    ]
    if is_msvc() and PY2:  # Only Python 2.7 & 3.2 Need VisualStudio 2008 (without stdint.h)
        openaes_include_dirs.append(join(EXT_SRC_DIR, 'include_win'))
    return openaes_include_dirs


def get_libraries(include_python_lib=False):
    libraries = []
    if sys.version_info[0] == 3 and sys.version_info[1] >= 5 and is_msvc():
        # https://stackoverflow.com/questions/32418766/c-unresolved-external-symbol-sprintf-and-sscanf-in-visual-studio-2015
        libraries = ['legacy_stdio_definitions']

    if is_msvc():
        link_py = 'python{0}{1}'.format(*sys.version_info[0:2])
    else:
        if PY2:
            link_py_fmt = 'python{version}'
        else:
            link_py_fmt = 'python{version}m'
        link_py = link_py_fmt.format(version=sysconfig.get_python_version())

    if include_python_lib:
        libraries.append(link_py)

    return libraries


def get_exe_link_args():
    ver = '%d.%d' % (sys.version_info[0], sys.version_info[1])
    if is_msvc() and (ver == '3.3' or ver == '3.4'):
        # For Fix Manifest error, https://bugs.python.org/issue4431
        return ['/MANIFEST']
    return None


include_dirs = get_include_dirs()
openaes_sources = [
    join(EXT_SRC_DIR, 'openaes', 'src', 'oaes_base64.c'),
    join(EXT_SRC_DIR, 'openaes', 'src', 'oaes_lib.c'),
]

ext_module = Extension(
    'pyconcrete._pyconcrete',
    include_dirs=include_dirs,
    libraries=get_libraries(include_python_lib=False),
    sources=[
        join(EXT_SRC_DIR, 'pyconcrete.c'),
        join(EXT_SRC_DIR, 'pyconcrete_module.c'),
    ] + openaes_sources,
)

exe_module = Extension(
    'pyconcrete',
    include_dirs=include_dirs,
    libraries=get_libraries(include_python_lib=True),
    define_macros=[('PYCONCRETE_VERSION', '"%s"' % version)],
    extra_link_args=get_exe_link_args(),
    sources=[
        join(EXE_SRC_DIR, 'pyconcrete_exe.c'),
        join(EXT_SRC_DIR, 'pyconcrete.c'),
    ] + openaes_sources,
)

# ================================================= setup ================================================= #


setup(
    name='pyconcrete',
    version=version,
    description='Protect your python script, encrypt it as .pye and decrypt when import it',
    long_description=readme,
    keywords='python source encryption obfuscation',
    author='Falldog',
    author_email='falldog7@gmail.com',
    url='https://github.com/Falldog/pyconcrete',
    license="Apache License 2.0",
    ext_modules=[ext_module],
    exe_modules=[exe_module],
    cmdclass={
        "build": BuildEx,
        "build_exe": BuildExe,
        "install": InstallEx,
        "test": TestEx,
    },
    scripts=[
        'pyconcrete-admin.py',
    ],
    packages=[
        'pyconcrete',
    ],
    package_dir={
        '': SRC_DIR,
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Security',
        'Topic :: Security :: Cryptography',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'License :: OSI Approved :: Apache Software License',
    ],
    distclass=ExeDistribution,
)



#!/usr/bin/env python
#
# Copyright 2015 Falldog Hsieh <falldog7@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import imp
import marshal
from os.path import join, exists, isdir

EXT_PY  = '.py'
EXT_PYC = '.pyc'
EXT_PYD = '.pyd'
EXT_PYE = '.pye'

__all__ = ["info"]

from . import _pyconcrete

info = _pyconcrete.info
encrypt_file = _pyconcrete.encrypt_file
decrypt_file = _pyconcrete.decrypt_file
decrypt_buffer = _pyconcrete.decrypt_buffer


class PyeLoader(object):
    def __init__(self, is_pkg, pkg_path, full_path):
        self.is_pkg = is_pkg
        self.pkg_path = pkg_path
        self.full_path = full_path
        with open(full_path, 'rb') as f:
            self.data = f.read()

    def new_module(self, fullname, path, package_path):
        m = imp.new_module(fullname)
        m.__file__ = path
        m.__loader__ = self
        if self.is_pkg:
            m.__path__ = [package_path]

        if "__name__" not in m.__dict__:
            m.__name__ = fullname

        return m

    def load_module(self, fullname):
        if fullname in sys.modules:  # skip reload by now ...
            return sys.modules[fullname]

        data = decrypt_buffer(self.data)  # decrypt pye

        self._validate_version(data)

        if sys.version_info >= (3, 7):
            # reference python source code
            # python/Lib/importlib/_bootstrap_external.py _code_to_timestamp_pyc() & _code_to_hash_pyc()
            # MAGIC + HASH + TIMESTAMP + FILE_SIZE
            magic = 16
        elif sys.version_info >= (3, 3):
            # reference python source code
            # python/Lib/importlib/_bootstrap_external.py _code_to_bytecode()
            # MAGIC + TIMESTAMP + FILE_SIZE
            magic = 12
        else:
            # load pyc from memory
            # reference http://stackoverflow.com/questions/1830727/how-to-load-compiled-python-modules-from-memory
            # MAGIC + TIMESTAMP
            magic = 8

        code = marshal.loads(data[magic:])

        m = self.new_module(fullname, self.full_path, self.pkg_path)
        sys.modules[fullname] = m
        exec(code, m.__dict__)
        return m

    def is_package(self, fullname):
        return self.is_pkg

    @staticmethod
    def _validate_version(data):
        magic = imp.get_magic()
        ml = len(magic)
        if data[:ml] != magic:
            import struct
            # convert little-endian byte string to unsigned short
            py_magic = struct.unpack('<H', magic[:2])[0]
            pye_magic = struct.unpack('<H', data[:2])[0]
            raise ValueError("Python version doesn't match with magic: python(%d) != pye(%d)" % (py_magic, pye_magic))


class PyeMetaPathFinder(object):
    def find_module(self, fullname, path=None):
        mod_name = fullname.split('.')[-1]
        paths = path if path else sys.path

        for trypath in paths:
            mod_path = join(trypath, mod_name)
            is_pkg = isdir(mod_path)
            if is_pkg:
                full_path = join(mod_path, '__init__' + EXT_PYE)
                pkg_path = mod_path
            else:
                full_path = mod_path + EXT_PYE
                pkg_path = trypath

            if exists(full_path):
                return PyeLoader(is_pkg, pkg_path, full_path)


sys.meta_path.insert(0, PyeMetaPathFinder())



#!/usr/bin/env python
# 
# Create on : 2017/01/18
#
# @author : Falldog
#
from os.path import join, abspath, dirname, pardir

DEFAULT_KEY = 'Falldog'

ROOT_DIR = abspath(join(dirname(__file__), pardir))
TEST_DIR = 'test'
SRC_DIR = join('src')
PY_SRC_DIR = join(SRC_DIR, 'pyconcrete')
EXT_SRC_DIR = join(SRC_DIR, 'pyconcrete_ext')
EXE_SRC_DIR = join(SRC_DIR, 'pyconcrete_exe')
SECRET_HEADER_PATH = join(EXT_SRC_DIR, 'secret_key.h')




#!/usr/bin/env python
#
# Copyright 2015 Falldog Hsieh <falldog7@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
import sys
import shutil
import atexit
import unittest
import tempfile
import subprocess
import py_compile
from os.path import dirname, abspath, join, exists
from src.config import SECRET_HEADER_PATH
try:
    from importlib import reload
except ImportError:
    pass


PY2 = sys.version_info[0] < 3
ROOT_DIR = abspath(join(dirname(__file__), '..'))
tmp_pyconcrete_dir = None
tmp_pyconcrete_exe = None


def to_bytes(s):
    if PY2:
        if isinstance(s, unicode):
            return s.encode('utf8')
        return s
    else:
        if isinstance(s, str):
            return s.encode('utf8')
        return s


def touch(file_path):
    open(file_path, 'a').close()


def build_tmp_pyconcrete(passphrase):
    global tmp_pyconcrete_dir, tmp_pyconcrete_exe
    if tmp_pyconcrete_dir:
        return tmp_pyconcrete_dir

    # remove secret key header, for testing on generate secret-key file
    if os.path.exists(SECRET_HEADER_PATH):
        os.remove(SECRET_HEADER_PATH)

    tmp_dir = tempfile.mkdtemp(prefix='pyconcrete_lib_')

    _ori_dir = os.getcwd()
    os.chdir(ROOT_DIR)
    try:
        # just build
        # force_option = '--force' if force else ''
        # subprocess.check_call('python setup.py build --passphrase=%s %s' % (passphrase, force_option), shell=True)

        cmd = (
            sys.executable,
            'setup.py',
            'install',
            '--passphrase=%s' % passphrase,
            '--install-base=%s' % tmp_dir,
            '--install-purelib=%s' % tmp_dir,
            '--install-platlib=%s' % tmp_dir,
            '--install-scripts=%s' % join(tmp_dir, 'scripts'),
            '--install-headers=%s' % join(tmp_dir, 'headers'),
            '--install-data=%s' % join(tmp_dir, 'data'),
            '--quiet',
        )
        subprocess.check_call(' '.join(cmd), shell=True)

        copy_pyconcrete_ext(tmp_dir)

        exe_name = 'pyconcrete.exe' if sys.platform == 'win32' else 'pyconcrete'
        tmp_pyconcrete_exe = join(tmp_dir, 'scripts', exe_name)
        tmp_pyconcrete_dir = tmp_dir
        print('build tmp pyconcrete at "%s"' % tmp_dir)

        if not exists(tmp_pyconcrete_exe):
            raise ValueError("can't find pyconcrete exe!")
    finally:
        os.chdir(_ori_dir)

    return tmp_pyconcrete_dir


def copy_pyconcrete_ext(tmp_dir):
    tmp_pyconcrete = join(tmp_dir, 'pyconcrete')
    for f in os.listdir(tmp_pyconcrete):
        if re.match('^_pyconcrete.*\.(so|dll|pyd)$', f):
            shutil.copy(
                join(tmp_pyconcrete, f),
                join(ROOT_DIR, 'src', 'pyconcrete')
            )
            break


def remove_tmp_pyconcrete():
    global tmp_pyconcrete_dir
    if tmp_pyconcrete_dir:
        shutil.rmtree(tmp_pyconcrete_dir)
        print('remove tmp pyconcrete at "%s"' % tmp_pyconcrete_dir)
        tmp_pyconcrete_dir = None
atexit.register(remove_tmp_pyconcrete)


def get_pyconcrete_env_path():
    """
    append tmp_pyconcrete_dir path into PYTHONPATH
    for subprocess execute script and import the pyconcrete we just builded
    """
    global tmp_pyconcrete_dir
    env = os.environ.copy()
    env.setdefault('PYTHONPATH', '')
    env['PYTHONPATH'] += os.pathsep + tmp_pyconcrete_dir
    return env


# ==================================== TestPyConcreteBase ==================================== #


class TestPyConcreteBase(unittest.TestCase):
    passphrase = 'Falldog'
    force_build = True

    def __init__(self, *argv, **argd):
        unittest.TestCase.__init__(self, *argv, **argd)

    @classmethod
    def setUpClass(cls):
        build_tmp_pyconcrete(cls.passphrase)

        cls.lib_dir = join(ROOT_DIR, 'src')
        cls._cls_sys_path = sys.path[:]
        sys.path.insert(0, cls.lib_dir)

        import pyconcrete

        # force to reload it, avoid to load installed module
        if not pyconcrete.__file__.startswith(cls.lib_dir):
            reload(pyconcrete)

    @classmethod
    def tearDownClass(cls):
        if cls._cls_sys_path:
            sys.path = cls._cls_sys_path
            cls._cls_sys_path = None

    def setUp(self):
        global tmp_pyconcrete_exe
        self.tmp_dir = tempfile.mkdtemp(prefix='pyconcrete_tmp_')
        self._sys_path = sys.path[:]
        sys.path.insert(0, self.tmp_dir)
        self._pyconcrete_exe = tmp_pyconcrete_exe

    def tearDown(self):
        if self._sys_path:
            sys.path = self._sys_path
            self._sys_path = None
        shutil.rmtree(self.tmp_dir)

    def lib_gen_py(self, py_code, py_filename, folder=None):
        """ folder = None -> use @_tmp_dir """
        if not folder:
            folder = self.tmp_dir
        self.assertTrue(py_filename.endswith('.py'))
        py_filepath = join(folder, py_filename)
        with open(py_filepath, 'wb') as f:
            f.write(to_bytes(py_code))
        return py_filepath

    def lib_gen_pyc(self, py_code, pyc_filename, folder=None, keep_py=False):
        """ folder = None -> use @_tmp_dir """
        if not folder:
            folder = self.tmp_dir
        self.assertTrue(pyc_filename.endswith('.pyc'))
        filename = os.path.splitext(pyc_filename)[0]
        py_filepath = join(folder, filename + '.py')
        pyc_filepath = join(folder, filename + '.pyc')

        # create .py
        with open(py_filepath, 'wb') as f:
            f.write(to_bytes(py_code))

        # create .pyc
        py_compile.compile(py_filepath, cfile=pyc_filepath)

        # remove files
        if not keep_py:
            os.remove(py_filepath)
        
        return pyc_filepath

    def lib_gen_pye(self, py_code, pye_filename, folder=None, keep_py=False, keep_pyc=False):
        """ folder = None -> use @_tmp_dir """
        if not folder:
            folder = self.tmp_dir
        self.assertTrue(pye_filename.endswith('.pye'))
        filename = os.path.splitext(pye_filename)[0]
        py_filepath = join(folder, filename + '.py')
        pyc_filepath = join(folder, filename + '.pyc')
        pye_filepath = join(folder, filename + '.pye')

        # create .py
        with open(py_filepath, 'wb') as f:
            f.write(to_bytes(py_code))

        # create .pyc
        py_compile.compile(py_filepath, cfile=pyc_filepath)

        # create .pye & remove .py & .pyc
        import pyconcrete
        pyconcrete.encrypt_file(pyc_filepath, pye_filepath)

        # remove files
        if not keep_py:
            os.remove(py_filepath)
        if not keep_pyc:
            os.remove(pyc_filepath)

        return pye_filepath

    def lib_compile_pyc(self, folder, remove_py=False):
        admin_path = join(ROOT_DIR, 'pyconcrete-admin.py')
        arg_remove_py = '--remove-py' if remove_py else ''
        subprocess.check_call('%s %s compile --source=%s --pyc %s' % (sys.executable, admin_path, folder, arg_remove_py), env=get_pyconcrete_env_path(), shell=True)

    def lib_compile_pye(self, folder, remove_py=False, remove_pyc=False):
        admin_path = join(ROOT_DIR, 'pyconcrete-admin.py')
        arg_remove_py = '--remove-py' if remove_py else ''
        arg_remove_pyc = '--remove-pyc' if remove_pyc else ''
        subprocess.check_call('%s %s compile --source=%s --pye %s %s' % (sys.executable, admin_path, folder, arg_remove_py, arg_remove_pyc), env=get_pyconcrete_env_path(), shell=True)





#!/usr/bin/env python
#
# Copyright 2015 Falldog Hsieh <falldog7@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

import base
import unittest


class TestConcrete(base.TestPyConcreteBase):
    def test_decrypt_exception(self):
        import pyconcrete
        print(pyconcrete.__file__)
        data = 'abc'
        self.assertLess(len(data), 16)
        
        with self.assertRaises(pyconcrete._pyconcrete.Error):
            pyconcrete.decrypt_buffer(data)

    def __do_encrypt_decrypt_file(self, py_code):
        py_filepath = self.lib_gen_py(py_code, 'test.py')
        pye_filepath = py_filepath + 'e'
        
        import pyconcrete
        pyconcrete.encrypt_file(py_filepath, pye_filepath)
        
        with open(pye_filepath, 'rb') as f:
            data = f.read()
        return pyconcrete.decrypt_buffer(data)

    def test_process_py_code_empty(self):
        py_code = b''
        res = self.__do_encrypt_decrypt_file(py_code)
        self.assertEqual(py_code, res)

    def test_process_py_code_large(self):
        py_code = ''
        for i in range(100):
            py_code += ('print("This is testing large py file ... %s")\r\n' % i)
        res = self.__do_encrypt_decrypt_file(py_code.encode('utf8'))
        self.assertEqual(py_code, res.decode('utf8'))

    def test_process_py_code_1_block(self):
        py_code = b'v=12345678901234'
        self.assertEqual(16, len(py_code))  # 1 block
        
        res = self.__do_encrypt_decrypt_file(py_code)
        self.assertEqual(py_code, res)

    def test_process_py_code_less_1_block(self):
        py_code = b'v=123456789'
        self.assertLess(len(py_code), 16)  # less than 1 block
        
        res = self.__do_encrypt_decrypt_file(py_code)
        self.assertEqual(py_code, res)

    def test_process_py_code_2_block(self):
        py_code = b'v1=12345678901\r\n'
        py_code += b'v2=12345678901\r\n'
        self.assertEqual(len(py_code), 32)  # 2 blocks
        
        res = self.__do_encrypt_decrypt_file(py_code)
        self.assertEqual(py_code, res)

if __name__ == '__main__':
    unittest.main()
